/**
 * @file server.c
 * @brief Implementación del driver SERVER.
 */
#include <server.h>

#include <wifly.h>
#include <uartio.h>

#include <FreeRTOS.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <portmacro.h>

#include <config.h>

/// Hostname donde se encuentra la aplicación web.
#define SERVER_SERVER_ADDRESS						"tfcjefernandez.appspot.com"

/// Puerto de la aplicación web.
#define SERVER_SERVER_PORT							80

/// Formato de envio de datos al servidor web.
#define SERVER_REQ_SENDSTATS__DID_SIP_SAPP_SDATA \
	"GET$/rest/stats?nid=1&ip=%d.%d.%d.%d&app=%s&data=%s"

/// Petición de datos del servidor.
#define SERVER_REQ_GETFEED__SHOST \
	"GET /rest/feed HTTP/1.1\r\nhost: %s\r\n"

/// Última IP obtenida.
uint32_t lastIp = 0;

/// Identificador del puerto UART usado por el WIFLY.
extern uint32_t uartId;

/// Nombre del punto de acceso a conectar.
const char *ssid;

/// Contraseña del punto de acceso.
const char *password;

/// Tipo de seguridad del punto de acceso.
WIFLY_SecurityType securityType;

/**
  * Inicializa el módulo de control del envio y recepción de datos al servidor.
  * Además de inicializar el módulo WiFly crea el mutex necesario para evitar
  * que las tareas hagan uso del módulo al mismo tiempo.
  * \param wiflyUartId Identificador de UART al que está conectado el módulo
  * WiFly.
  * \param _ssid Nombre del punto de acceso al que queremos conectar.
  * \param _securityType Define el tipo de seguridad que tenemos activado en
  * el punto de acceso.
  * \param _password Contraseña del punto de acceso.
  */
void SERVER_Initialize(uint32_t wiflyUartId, const char *_ssid,
		WIFLY_SecurityType _securityType, const char *_password)
{
	LOG("Initializating Server library");

	WIFLY_Initialize(wiflyUartId);
	ssid = _ssid;
	securityType = _securityType;
	password = _password;
	//WIFLY_ConnectToWLAN(ssid, securityType, password);
}

/**
  * Envía un dato al servidor para hacer el registro de un dato.
  * \param application Nombre de la aplicación que está enviando los datos.
  * \param value La cadena de texto correspondiente del valor que queremos
  * enviar al servidor. El formato printf que se debería usar es \e "%.2f".
  * \return true si se han enviado los datos correctamente, false de otro modo.
  */
bool SERVER_SendStats(const char *application, const char *value)
{
	bool retval;
	char remote[128];

	LOG1("Sending value %d", value);

	if(WIFLY_GetIP() == 0)
		retval = WIFLY_ConnectToWLAN(ssid, securityType, password);
	else
		retval = true;
	if(retval)
	{
		if(0 == lastIp)
			lastIp = WIFLY_GetIP();

		sprintf(remote, SERVER_REQ_SENDSTATS__DID_SIP_SAPP_SDATA,
				(lastIp & 0xFF000000) >> 24,
				(lastIp & 0xFF0000) >> 16,
				(lastIp & 0xFF00) >> 8,
				(lastIp & 0xFF),
				application,
				value);

		retval = WIFLY_Open(SERVER_SERVER_ADDRESS, SERVER_SERVER_PORT, 1, remote);
		WIFLY_Close(true);
	}
	return retval;
}

/**
  * Recibe la configuración desde el servidor.
  * \param settings Puntero al objeto global que contiene la configuración.
  * \return El dato devuelto desde el servidor o -1 en caso de haber fallado.
  */
void SERVER_GetSettings(SERVER_Settings *settings)
{
	bool result, initialized = false;

	if(WIFLY_GetIP() == 0)
		result = WIFLY_ConnectToWLAN(ssid, securityType, password);
	else
		result = true;
	if(result)
	{
		result = WIFLY_Open(SERVER_SERVER_ADDRESS, SERVER_SERVER_PORT, 0, "0");
		if(result)
		{
			char *pChar;
			char dataInit[] = "\r\n\r\n";
			char *currentSetting;

			UARTIO_Printfln(uartId, dataInit, SERVER_REQ_GETFEED__SHOST,
					SERVER_SERVER_ADDRESS);
			pChar = UARTIO_Str(uartId, 0, dataInit) + strlen(dataInit);

			currentSetting = "Interval: ";
			pChar = strstr(pChar, currentSetting) + strlen(currentSetting);
			settings->interval = atoi(pChar);

			currentSetting = "TempMax: ";
			pChar = strstr(pChar, currentSetting) + strlen(currentSetting);
			settings->temperatureMax = atoi(pChar);

			currentSetting = "TempMin: ";
			pChar = strstr(pChar, currentSetting) + strlen(currentSetting);
			settings->temperatureMin = atoi(pChar);

			currentSetting = "HumiMax: ";
			pChar = strstr(pChar, currentSetting) + strlen(currentSetting);
			settings->humidityMax = atoi(pChar);

			currentSetting = "HumiMin: ";
			pChar = strstr(pChar, currentSetting) + strlen(currentSetting);
			settings->humidityMin = atoi(pChar);

			initialized = true;

			WIFLY_Close(false);
		}
	}
	//WIFLY_Sleep();
	if(settings->initialized == false)
		settings->initialized = initialized;
	LOG("Received settings:\r\n"
			"Interval: %d\r\n"
			"Temperature Range: %d°C - %d°C\r\n"
			"Humidity Range: %d%% - %d%%",
			settings->interval,
			settings->temperatureMin, settings->temperatureMax,
			settings->humidityMin, settings->humidityMax);
}

