/**
 * @file sht1x.c
 * @brief Implementación del driver SHT1X.
 */
#include <sht1x.h>

#include <FreeRTOS.h>
#include <LPC17xx.h>

#include <math.h>
#include <stdbool.h>

#include <sleep.h>
#include <uartio.h>

#include <config.h>

/// Pin que hará de señal de reloj.
uint8_t _sckPin = -1;

/// Pin que se usará para enviar y recibir datos.
uint8_t _dataPin = -1;

/// Tiempo de espera de reloj.
const uint32_t Tsck = 2;

/// Comandos disponibles por el driver.
typedef enum _SHT1X_Command
{
	/// Comando para medir la temperatura.
	SHT1X_MeasureTemperature = 0b00000011,
	/// Comando para medir la humedad.
	SHT1X_MeasureRelativeHumidity = 0b00000101
} SHT1X_Command;

/**
 * Inicializa el sensor de temperatura y humedad.
 * \param sckPin Pin en el que está conectada la señal de clock.
 * \param dataPin Pin en el que está conectada la señal de datos.
 */
void SHT1X_Initialize(uint8_t sckPin, uint8_t dataPin)
{
	LOG("Initializing PINs: SCK = %d, DATA = %d", sckPin, dataPin);

	_sckPin = sckPin;
	_dataPin = dataPin;

	SLEEP_PasiveMSSleep(15);
	LPC_GPIO0->FIODIR |= (1 << sckPin); // Set output
	LPC_PINCON->PINMODE0 |= (2 << (dataPin * 2));
	LPC_PINCON->PINSEL0 &= ~((3 << (dataPin * 2)) | (3 << (sckPin * 2)));
}

/**
 * Especifica si el pin data está recibiendo o enviando datos.
 * \param output Especifica la dirección.
 */
inline void SHT1X_SetDataDirection(bool output)
{
	if(output)
		LPC_GPIO0->FIODIR |= 1 << _dataPin;
	else
		LPC_GPIO0->FIODIR &= ~(1 << _dataPin);
}

/**
 * Pone un 1 en la patilla del pin de datos
 */
void SHT1X_DataSet() { LPC_GPIO0->FIOSET |= 1 << _dataPin; }
/**
 * Pone un 0 en la patilla del pin de datos.
 */
void SHT1X_DataClear() { LPC_GPIO0->FIOCLR |= 1 << _dataPin; }

/**
 * Envia una señal de reloj y llama a la función especificada en medio de la
 * señal de reloj.
 * \param fnData Función que será llamada.
 */
void SHT1X_Clock(void (*fnData)(void))
{
	LPC_GPIO0->FIOSET |= 1 << _sckPin;
	SLEEP_PasiveMSSleep(NULL != fnData ? Tsck / 2 : Tsck);
	if(NULL != fnData)
	{
		fnData();
		SLEEP_PasiveMSSleep(Tsck / 2);
	}
	LPC_GPIO0->FIOCLR |= 1 << _sckPin;
	SLEEP_PasiveMSSleep(Tsck);
}

/**
 * Marca el inicio de la transmisión.
 */
inline void SHT1X_TransmissionStart()
{
	// Transmission start sequence
	SHT1X_Clock(SHT1X_DataClear);
	SHT1X_Clock(SHT1X_DataSet);

	SHT1X_DataClear();
	SLEEP_PasiveMSSleep(Tsck);
}

/**
 * Envía una señal de reloj llamando a la función dada en el flanco de subida.
 * \param fnData Función que será llamada.
 */
void SHT1X_ClockCommand(void (*fnData)(void))
{
	if(NULL != fnData)
		fnData();
	SLEEP_PasiveMSSleep(Tsck / 2);
	LPC_GPIO0->FIOSET |= 1 << _sckPin;
	SLEEP_PasiveMSSleep(Tsck);
	LPC_GPIO0->FIOCLR |= 1 << _sckPin;
	SLEEP_PasiveMSSleep(Tsck / 2);
}

/**
 * Pone el pin de datos en modo de entrada de datos.
 */
void SHT1X_SetDataDirectionToInput() { SHT1X_SetDataDirection(false); }

/**
 * Envía un comando al sensor, para indicar si leerá temperatura o humedad.
 * \param command Indica el comando que se enviará.
 * \return El valor devuelto por el sensor.
 */
inline int16_t SHT1X_SendCommand(SHT1X_Command command)
{
	int i;
	int16_t value = 0;

	SHT1X_SetDataDirection(true);
	SLEEP_PasiveMSSleep(5);

	// Initial state
	SHT1X_DataSet();
	LPC_GPIO0->FIOCLR |= 1 << _sckPin;
	SLEEP_PasiveMSSleep(10);

	// Transmission start sequence
	SHT1X_TransmissionStart();

	// Command
	for(i = 7; i >= 0; i--)
	{
		const bool value = command & (1 << i);
		SHT1X_ClockCommand(value ? SHT1X_DataSet : SHT1X_DataClear);
	}

	// ACK
	SHT1X_Clock(SHT1X_SetDataDirectionToInput);

	// Measurement ~80ms
	while(!(LPC_GPIO0->FIOPIN & (1 << _dataPin)));
	while(LPC_GPIO0->FIOPIN & (1 << _dataPin));

	for(i = 0; i < 16; i++)
	{
		value += ((LPC_GPIO0->FIOPIN & (1 << _dataPin)) >> 10) << (15 - i);
		SHT1X_Clock(NULL);
		if(i == 7) // Send ACK
		{
			SHT1X_SetDataDirection(true);
			SHT1X_ClockCommand(SHT1X_DataClear);
			SHT1X_SetDataDirection(false);
		}
	}
	return value;
}

/**
 * Lee la temperatura del sensor y la devuelve en la unidad dada.
 * \param unit Unidad de la temperatura, puede ser Celsius, Fahrenheit o Kelvin.
 * \return Devuelve la temperatura en la unidad indicada.
 */
float SHT1X_ReadTemperature(SHT1X_TemperatureUnit unit)
{
	int16_t value = SHT1X_SendCommand(SHT1X_MeasureTemperature);
	float degrees = 0;
	switch(unit)
	{
	case SHT1X_Fahrenheit:
		degrees += value * 0.016 - 39.3; break;
	case SHT1X_Kelvin:
		degrees = 273.15;
	case SHT1X_Celsius:
		degrees += value * 0.01 - 39.6; break;
	}
	return degrees;
}

/**
 * Lee la humedad del sensor y calcula la humedad relativa.
 * \param temperature Temperatura medida.
 * \return Humedad relativa medida.
 */
float SHT1X_ReadRelativeHumidity(float temperature)
{
	const float c1 = -2.0468, c2 = 0.0367, c3 = -0.0000015955, t1 = 0.01,
			t2 = 0.00008;

	int16_t value = SHT1X_SendCommand(SHT1X_MeasureRelativeHumidity);
	float linearHumidity = c1 + c2 * value + c3 * value * value;
	float correctedHumidity = (temperature - 25) * (t1 + t2 * value)
			+ linearHumidity;
	return correctedHumidity;
}
