/**
 * @file uartio.c
 * @brief Implementación del driver UARTIO.
 */
#include <uartio.h>

#include <uart.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <LPC17xx.h>

#include <FreeRTOS.h>
#include <task.h>
#include <semphr.h>

#include <config.h>
#include <sleep.h>

extern volatile uint8_t UART0Buffer[BUFSIZE], UART1Buffer[BUFSIZE],
	UART3Buffer[BUFSIZE];
extern volatile uint32_t UART0Count, UART1Count, UART3Count;

/// Estructura para encapsular los punteros a los diferentes puerto UART.
typedef struct {
	/// Indica si se encuentra en uso.
	bool used;
	/// Puntero al buffer del puerto UART.
	volatile uint8_t* buffer;
	/// Puntero al contador de datos disponibles en el buffer.
	volatile uint32_t* count;
	/// Puntero al IER del puerto UART.
	volatile uint32_t* IER;
	/// Mutex del puerto UART, para evitar 2 accesos al mismo tiempo al mismo puerto UART.
	xSemaphoreHandle mutex;
} UARTPOINTERS;

/// Colección de definiciones de punteros UART.
static UARTPOINTERS UARTpointers[] = {
	{false, UART0Buffer, &UART0Count, &LPC_UART0->IER, NULL},
	{false, UART1Buffer, &UART1Count, &LPC_UART1->IER, NULL},
	{false, NULL, NULL, NULL, NULL},
	{false, UART3Buffer, &UART3Count, &LPC_UART3->IER, NULL}
};

/**
  * Inicializa el puerto UART para enviar y/o recibir datos.
  * \param uartId Identificador del puerto UART a usar.
  * \param baudRate Velocidad a la que transmitiremos y recibiremos datos.
  * \return true Si la inicialización ha sido correcta, false en otro caso.
  */
bool UARTIO_Init(uint32_t uartId, uint32_t baudRate)
{
	bool retval = false;
	UARTPOINTERS *pointers = &UARTpointers[uartId];

	if(/*!pointers->used && */UARTInit(uartId, baudRate))
	{
		pointers->used = true;
		pointers->mutex = xSemaphoreCreateMutex();
		retval = pointers->mutex != NULL;
	}
	return retval;
}

/**
  * Función interna que envia datos a través del puerto UART especificado. Se le
  * puede especificar una cadena de texto en caso de que queremos quedar a la
  * espera hasta recibir una respuesta buscada.
  * \param uartId Identificador del puerto UART.
  * \param checkString Cadena de texto opcional para que en el caso de necesitar
  * quedar a la espera de un dato la función se quede a la espera de él.
  * \param format Cadena de texto con el formato de la cadena a enviar.
  * \param args Lista de argumentos que se que se sustituirán sobre el format
  * definido en la variable format.
  * \return La longitud de la cadena en caso de que no se haya definido
  * cadena de comprobación ó se haya recibido la confirmación, 0 en otro caso.
  */
int UARTvprintf(int uartId, const char *checkString, const char *format,
		va_list *args)
{
	char message[BUFSIZE];
	char *pChar;
	int len;
	UARTPOINTERS *pointers = &UARTpointers[uartId];

	xSemaphoreTake(pointers->mutex, portMAX_DELAY);
	len = vsnprintf(message, BUFSIZE, format, *args);
	*pointers->IER = IER_THRE | IER_RLS;				/* Disable RBR */
	memset((void*)UARTpointers[uartId].buffer, 0, *UARTpointers[uartId].count);
    *pointers->count = 0;
	*pointers->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
	UARTSend(uartId, (uint8_t *)message, len);
	if(uartId == 0)
		LOG1("Sent: %s", message);
	if(NULL != checkString)
	{
		pChar = UARTIO_Str(uartId, 5000, checkString);
		len = pChar != NULL ? len : 0;
		LOG1("%s %s",
				checkString, len > 0 ? "found" : "not found");
	}
	xSemaphoreGive(pointers->mutex);
	return len;
}

/**
  * Envia datos a través del puerto UART especificado. Se le puede especificar
  * una cadena de texto en caso de que queremos quedar a la
  * espera hasta recibir una respuesta buscada. No envia al final de la cadena
  * CR+LF.
  * \param uartId Identificador del puerto UART.
  * \param checkString Cadena de texto opcional para que en el caso de necesitar
  * quedar a la espera de un dato la función se quede a la espera de él.
  * \param format Cadena de texto con el formato de la cadena a enviar.
  * \param ... Lista de argumentos que se que se sustituirán sobre el format
  * definido en la variable format.
  * \return La longitud de la cadena en caso de que no se haya definido
  * cadena de comprobación ó se haya recibido la confirmación, 0 en otro caso.
  */
int UARTIO_Printf(int uartId, const char *checkString, const char *format, ...)
{
	va_list args;
	int len;

	va_start(args, format);
	len = UARTvprintf(uartId, checkString, format, &args);
	va_end(args);
	return len;
}

/**
  * Envia datos a través del puerto UART especificado. Se le puede especificar
  * una cadena de texto en caso de que queremos quedar a la
  * espera hasta recibir una respuesta buscada. Envia al final de la cadena
  * CR+LF.
  * \param uartId Identificador del puerto UART.
  * \param checkString Cadena de texto opcional para que en el caso de necesitar
  * quedar a la espera de un dato la función se quede a la espera de él.
  * \param format Cadena de texto con el formato de la cadena a enviar.
  * \param ... Lista de argumentos que se que se sustituirán sobre el format
  * definido en la variable format.
  * \return La longitud de la cadena en caso de que no se haya definido
  * cadena de comprobación ó se haya recibido la confirmación, 0 en otro caso.
  */
int UARTIO_Printfln(int uartId, const char *checkString, const char *format, ...)
{
	va_list args;
	int len = strlen(format);
	char newline[] = "\r\n";
	char *newFormat = (char*)malloc(len + strlen(newline) - 1);

    sprintf(newFormat, "%s%s", format, newline);
	va_start(args, format);
	len = UARTvprintf(uartId, checkString, newFormat, &args);
	va_end(args);
	free(newFormat);
	return len;
}

/**
  * Obtiene los datos que hay en el buffer del puerto UART especificado.
  * \param uartId Identificador del puerto UART.
  * \param delayms Milisegundos de espera antes de comenzar a recibir los datos.
  * \param message Puntero a una cadena donde almacenaremos los datos extraídos
  * del buffer UART.
  * \return Datos recibidos desde el buffer UART.
  */
#ifdef USE_VSSCANF
int UARTIO_Scanf(int uartId, int delayms, char *format, ...)
#else
int UARTIO_Scanf(int uartId, int delayms, char *message)
#endif
{
	uint32_t oldCount, newCount = -1;
#ifdef USE_VSSCANF
	int retval;
	va_list args;

	va_start(args, format);
#endif
	SLEEP_PasiveMSSleep(delayms);
    do {
        oldCount = newCount;
        SLEEP_PasiveMSSleep(5);
        newCount = *UARTpointers[uartId].count;
    } while(oldCount != newCount);

	*UARTpointers[uartId].IER = IER_THRE | IER_RLS;				/* Disable RBR */

	UARTpointers[uartId].buffer[newCount] = '\0';
#ifdef USE_VSSCANF
	retval = vsscanf((char*)UARTpointers[uartId].buffer, format, args);
#else
	strncpy(message, (char*)UARTpointers[uartId].buffer, newCount + 1);
#endif
	message[*UARTpointers[uartId].count] = '\0';
	*UARTpointers[uartId].count = 0;

	*UARTpointers[uartId].IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
#ifdef USE_VSSCANF
	return retval;
#else
	return newCount;
#endif
}

/**
  * Busca una cadena y espera por una cadena dentro del buffer UART.
  * \param uartId Identificador del puerto UART.
  * \param timeoutms Milisegundos de espera para tratar de encontrar el dato
  * esperado.
  * \param string Cadena de texto a buscar dentro del buffer UART.
  * \return Posición con la coincidencia de la cadena de texto buscada, NULL en
  * caso de que no se haya encontrado y se haya terminado el tiempo de espera.
  */
char* UARTIO_Str(int uartId, int timeoutms, const char *string)
{
	UARTPOINTERS *pointers = &UARTpointers[uartId];
	uint32_t oldCount, newCount = -1;
	char* pChar = NULL;
	do {
		oldCount = newCount;
		if(NULL == pChar)
			pChar = strstr((char*)pointers->buffer, string);
		SLEEP_PasiveMSSleep(3);
		newCount = *pointers->count;
	} while((timeoutms-- >= 0 && pChar == NULL) || newCount != oldCount);
	return pChar;
}
