/**
 * @file wifly.c
 * @brief Implementación del driver WIFLY.
 */
#include <wifly.h>

#include <LPC17xx.h>
#include <FreeRTOS.h>

#include <type.h>
#include <task.h>
#include <uart.h>
#include <stdio.h>
#include <string.h>
#include <uartio.h>
#include <stdlib.h>
#include <semphr.h>

#include <config.h>

#include <sleep.h>

#define RST_WIFLY_PIN 22
#define WF_COM_GET_IP           				"get ip a"
#define WF_COM_GET_CHANNEL      				"get wlan channel"
#define WF_COM_GET_RSSI         				"show rssi"
#define WF_COM_GET_TIME         				"show time"
#define WF_COM_GET_TX_POWER						"get wlan"
#define WF_COM_SET_WLAN_PHRASE__SKEY  			"set wlan phrase %s"
#define WF_COM_SET_WEP_KEY__SKEY      			"set wlan key %s"
#define WF_COM_SET_TX_POWER__DVALUE				"set wlan tx %d"
#define WF_COM_JOIN__SSSID         				"join %s"
#define WF_COM_PING__SADDRESS					"ping %s"
#define WF_COM_OPEN__SADDRESS_DPORT				"open %s %d"

#define WF_RES_AOK								"AOK"

char message[BUFSIZE];
extern volatile uint32_t UART0Count;

uint32_t uartId = -1;
xSemaphoreHandle mutex = NULL;

bool WIFLY_EnterCommandMode();
void WIFLY_ExitCommandMode();

/**
  * Inicializa el dispositivo WiFly conectado al puerto UART especificado.
  * \param _uartId Identificador del puerto UART al que está conectado el WiFly.
  * \return Devuelve \e true en el caso de que el dispositivo WiFly no esté
  * inicializado, \e false en el caso contrario.
  */
bool WIFLY_Initialize(uint32_t _uartId)
{
	if(uartId == -1)
		uartId = _uartId;

	if(mutex)
		return false;
	mutex = xSemaphoreCreateMutex();
	xSemaphoreTake(mutex, portMAX_DELAY);

	// 1
	LOG("Initializing UART%d for WiFly", uartId);
	UARTIO_Init(uartId, 9600); /* baud rate setting */

	// 2
	SLEEP_PasiveMSSleep(3000);

	// 3
	LOG1("Sending RESET");
	LPC_PINCON->PINSEL1 &= (~(3 << 12));
	LPC_GPIO0->FIODIR |= (1 << RST_WIFLY_PIN);
	// it made reset at Wifly
	LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
	//sleep(1); // min 160us
	SLEEP_PasiveMSSleep(1);
	LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;

	// 4
	UARTIO_Str(uartId, 4000, "WiFly Ver ");
	UARTIO_Scanf(uartId, 0, message);

	// 5
	LOG1("Received message: %s", message);
	SLEEP_PasiveMSSleep(1000);
	xSemaphoreGive(mutex);
	return true;
}

/**
  * Pone el WiFly a la espera de recibir comandos. Además bloquea el mutex para
  * que no se ejecute ninguna otra acción sobre el dispositivo WiFly.
  * \return true en el caso de que se haya podido enviar y confirmar el comando.
  */
bool WIFLY_EnterCommandMode() {
	int result = xSemaphoreTake(mutex, portMAX_DELAY) == pdTRUE;

	if(result)
	{
		LOG2("Mutex 0x%08x locked", mutex);
		// Este exit enviado es por si hemos puesto el WiFly a dormir o si por
		// algún error anterior no hemos salido correctamente del command mode.
		UARTIO_Printfln(uartId, NULL, "exit");
		SLEEP_PasiveMSSleep(250);
		result = UARTIO_Printf(uartId, "CMD", "$$$");
		if (result)
			LOG1("Received CMD");
		else
			LOG1("CMD not received");
	}
	return result;
}

/**
  * Hace salir al WiFly del modo de recepción de comandos. Y libera el mutex.
  */
void WIFLY_ExitCommandMode()
{
	LOG1("Exit from command mode requested");
	UARTIO_Printfln(uartId, NULL, "exit");
	xSemaphoreGive(mutex);
	LOG2("Mutex 0x%08x released", mutex);
}

/**
  * Conecta el WiFly al punto de acceso indicado con los datos de conexión
  * indicados.
  * \param ssid Nombre del punto de acceso al que nos queremos conectar.
  * \param secType Tipo de seguridad que usa el punto de acceso al que se
  * conectará el WiFly.
  * \param password Contraseña necesaria para conectar al punto de acceso.
  * \return true en el caso de que se haya podido conectar al punto de acceso,
  * false en otro caso.
  */
bool WIFLY_ConnectToWLAN(const char* ssid, WIFLY_SecurityType secType,
		const char* password) {
	char *keyCmd = secType == WIFLY_ST_WEP ? WF_COM_SET_WEP_KEY__SKEY
					: WF_COM_SET_WLAN_PHRASE__SKEY;
	char *pChar;

	if(false == WIFLY_EnterCommandMode()) return false;

	UARTIO_Printfln(uartId, NULL, keyCmd, password);
	pChar = UARTIO_Str(uartId, 1000, WF_RES_AOK);

	// Connecting...
	UARTIO_Printfln(uartId, NULL, WF_COM_JOIN__SSSID, ssid);
	pChar = UARTIO_Str(uartId, 60000, "Listen on 2000");
	LOG("Connected to %s", ssid);
	WIFLY_ExitCommandMode();
	return NULL != pChar;
}

/**
  * Obtiene el la IP del WiFly. Es necesario estar en el modo comandos.
  * \return Un entero sin signo de 32 que contiene la dirección IP. Cada octeto
  * se corresponde a uno de los octetos que forman la dirección IP.
  */
uint32_t WIFLY_GetIP() {
	uint32_t ip = 0;
	char *pChar, octet[4];
	int i;

	if(false == WIFLY_EnterCommandMode()) return false;

	UARTIO_Printfln(uartId, NULL, WF_COM_GET_IP);
	UARTIO_Scanf(uartId, 100, message);

	pChar = strchr(message, '\n');

	for (i = 0; i < 4; i++)
		octet[i] = (char) strtol(pChar + sizeof(char), &pChar, 10);
	ip = (((((octet[0] << 8) + octet[1]) << 8) + octet[2]) << 8) + octet[3];
	WIFLY_ExitCommandMode();
	return ip;
}

/**
  * Devuelve el canal que está usando el WiFly actualmente. Es necesario estar
  * en modo comando.
  * \return El canal en uso.
  */
uint8_t WIFLY_GetChannel()
{
	uint8_t channel = 0;
	char preffix[] = "TxPower=";
	char *pChannel;

	if(false == WIFLY_EnterCommandMode()) return false;

	UARTIO_Printfln(uartId, NULL, WF_COM_GET_CHANNEL);
	UARTIO_Scanf(uartId, 50, message);

	pChannel = strstr(message, preffix) + strlen(preffix);
	channel = strtol(pChannel, &pChannel, 10);

	WIFLY_ExitCommandMode();
	return channel;
}

/**
  * Devuelve el valor actual de RSSI. Es necesario estar en modo comando.
  * \return Valor RSSI actual.
  */
int8_t WIFLY_GetRSSI()
{
	char preffix[] = "RSSI=";
	char *pChar;
	int8_t rssi = 0;

	if(false == WIFLY_EnterCommandMode()) return rssi;

	UARTIO_Printfln(uartId, NULL, WF_COM_GET_RSSI);
	UARTIO_Scanf(uartId, 50, message);

	pChar = strstr(message, preffix) + strlen(preffix);
	if (*pChar == '(')
		pChar++;
	rssi = strtol(pChar, &pChar, 10);
	WIFLY_ExitCommandMode();
	return rssi;
}

/**
  * Devuelve el tiempo que ha transcurrido desde que el WiFly se ha encendido.
  * Es necesario encontrarse en modo comando.
  * \return Tiempo que lleva encendido el WiFly.
  */
uint32_t WIFLY_GetUptime()
{
	char preffix[] = "UpTime=";
	char *pChar;
	uint32_t uptime = 0;

	if(false == WIFLY_EnterCommandMode()) return uptime;

	UARTIO_Printfln(uartId, NULL, WF_COM_GET_TIME);
	UARTIO_Scanf(uartId, 1000, message);

	pChar = strstr(message, preffix) + strlen(preffix);
	uptime = strtol(pChar, &pChar, 10);

	WIFLY_ExitCommandMode();
	return uptime;
}

/**
  * Realiza un ping a la dirección dada. Es necesario encontrarse en modo
  * comando.
  * \param address Dirección a la que realizaremos el ping.
  * \return Tiempo de respuesta del comando ping en milisegundos. En caso de
  * fallo se devolverá -1.
  */
uint32_t WIFLY_Ping(char *address)
{
	char *pChar;
	char preffix[] = "time=";
	uint32_t time = -1; // in miliseconds

	if(false == WIFLY_EnterCommandMode()) return time;

	UARTIO_Printfln(uartId, NULL, WF_COM_PING__SADDRESS, address);

	pChar = UARTIO_Str(uartId, 500, preffix) + strlen(preffix);
	time = atof(pChar);

	WIFLY_ExitCommandMode();
	return time;
}

/**
  * Devuelve el valor de la potencia de transmisión. Es necesario estar en modo
  * comando.
  * \return El valor actual de potencia de transmisión.
  */
uint8_t WIFLY_GetTxPower()
{
	int result;
	char preffix[] = "TxPower=";
	uint8_t retval = -1;

	if(false == WIFLY_EnterCommandMode()) return retval;

	result = UARTIO_Printfln(uartId, preffix, WF_COM_GET_TX_POWER);
	if(result > 0)
	{
		char *pChar = UARTIO_Str(uartId, 0, preffix) + strlen(preffix);
		retval = (uint8_t)atoi(pChar);
	}

	WIFLY_ExitCommandMode();
	return retval;
}

/**
  * Cambia el valor de la potencia de transmisión. Es necesario estar en modo
  * comando.
  * \param value El nuevo valor de potencia de transmisión.
  */
bool WIFLY_SetTxPower(uint8_t value)
{
	bool retval;
	if(false == WIFLY_EnterCommandMode()) return false;

	retval = UARTIO_Printfln(uartId, WF_RES_AOK, WF_COM_SET_TX_POWER__DVALUE,
			value) > 0;
	WIFLY_ExitCommandMode();
	return retval;
}

/**
  * Realiza una conexión HTTP contra un servidor dado. Se puede configurar para
  * realizar un envío de datos automático al realizar la conexión o activo
  * dejando la conexión abierta tras realizar la conexión para enviar los datos
  * manualmente. Es necesario estar en modo comando.
  * \param address Dirección a la que nos queremos conectar.
  * \param port Puerto al que se quiere conectar.
  * \param format 0 si queremos enviar los datos manualmente tras abrir la
  * conexión o 1 en el caso de que queremos que se envíen los datos
  * especificados en el parámetro remote tras establecer la conexión.
  * \param remote Cadena a enviar automáticamente una vez realizada la conexión
  * en caso de que queramos enviar datos manualmente enviar en este parámetro
  * "".
  * \return true en caso de que se haya realizado la conexión correctamente,
  * false en otro caso.
  */
bool WIFLY_Open(char *address, uint32_t port, uint8_t format,
		const char *remote)
{
	int result;

	if(false == WIFLY_EnterCommandMode()) return false;

	UARTIO_Printfln(uartId, WF_RES_AOK, "set ip proto 18");
	UARTIO_Printfln(uartId, WF_RES_AOK, "set dns name %s", address);
	UARTIO_Printfln(uartId, WF_RES_AOK, "set ip host 0");
	UARTIO_Printfln(uartId, WF_RES_AOK, "set ip remote %d", port);
	UARTIO_Printfln(uartId, WF_RES_AOK, "set option format %d", format);
	UARTIO_Printfln(uartId, WF_RES_AOK, "set com remote %s", remote);
	//UARTprintfln(uartId, WF_RES_AOK, "set com idle 3");

	result = UARTIO_Printfln(uartId, format > 0 ? NULL : "*OPEN*",
			WF_COM_OPEN__SADDRESS_DPORT, address, port);
	UARTIO_Scanf(uartId, 10, message);
	LOG1("OPEN Result: %s", message);
	SLEEP_PasiveMSSleep(100);
	return result > 0;
}

/**
  * Libera el mutex del WiFly una vez se ha realizado la acción de open y el
  * envío de datos (si es necesario). Además si se le indica se pone a la espera
  * del aviso de conexión cerrada.
  * \param waitForClose Indica si es necesario esperar por el close emitido por
  * el WiFly.
  */
void WIFLY_Close(bool waitForClose)
{
	if(waitForClose)
	{
		char *data;
		LOG1("Waiting for CLOSE message");
		if((data = UARTIO_Str(uartId, 5000, "CLOS")))
			LOG1("CLOSE message found %s", data);
		else
			LOG1("CLOSE message not found");
		//xSemaphoreGive(mutex);
		WIFLY_ExitCommandMode();
	}
	else
	{
		SLEEP_PasiveMSSleep(4000);
		//xSemaphoreGive(mutex);
		WIFLY_ExitCommandMode();
	}
	LOG2("Mutex 0x%08x released", mutex);
}

/**
 * Pone el WiFly a dormir, para que consuma menos energía.
 */
void WIFLY_Sleep()
{
	LOG1("WiFly going to sleep");
	WIFLY_EnterCommandMode();
	UARTIO_Printfln(uartId, NULL, "sleep");
	xSemaphoreGive(mutex);
}
