/**
 * @file tasksensor.c
 * @brief Archivo que contiene la implementación de la función ::TASK_Sensor.
 */
#include "common.h"
#include <stdbool.h>
#include <config.h>
#include <sht1x.h>

/**
 * Se encarga del manejo del sensor de temperatura y humedad. Durante la
 * toma de datos del sensor se activa el led ::LEDID_MEASURING_MASK que indica
 * que el microcontrolador se está comunicando con el sensor SHT15.
 *
 * Cuando la lectura del sensor ha finalizado la tarea se queda esperando un
 * tiempo definido por SERVER_Settings#interval.
 *
 * En el caso de que la toma de datos haya detectado que el valor de temperatura
 * no se encuentra dentro de los márgenes de seguridad
 * SERVER_Settings#temperatureMin y SERVER_Settings#temperatureMax esta tarea
 * encenderá el led ::LEDID_ALARM_TEMPERATURE_MASK hasta que una siguiente toma
 * de temperatura vuelva a estar dentro de los márgenes de seguridad.
 *
 * En el caso de que la toma de datos haya detectado que el valor de humedad
 * no se encuentra dentro de los márgenes de seguridad
 * SERVER_Settings#humidityMin y SERVER_Settings#humidityMax esta tarea
 * encenderá el led ::LEDID_ALARM_HUMIDITY_MASK hasta que una siguiente toma
 * de temperatura vuelva a estar dentro de los márgenes de seguridad.
 * \param pvParameters Contiene los parámetros que utilizará la tarea.
 */
void TASK_Sensor(void *pvParameters)
{
	unsigned long stackHWM;
	QueueData temperatureData = {QueueDataType_Temperature, 0};
	QueueData humidityData = {QueueDataType_Humidity, 0};
	QueueData data;
	bool temperatureAlarm = false, humidityAlarm = false;

	UNUSED(pvParameters);

	LOG("Starting task %s", __FUNCTION__);

	// Esperamos hasta que se haya obtenido la configuración de Internet
	while(!settings.initialized) SLEEP_PasiveMSSleep(5000);

	RTCInit();
	SHT1X_Initialize(11, 10);
	while(true)
	{
		LED_Switch(LEDID_MEASURING_MASK, true);
		// Tratamos datos de temperatura
		temperatureData.value = SHT1X_ReadTemperature(SHT1X_Celsius);
		LOG("Current temperature is %.2f°C", temperatureData.value);
		xQueueSend(xQueue, &temperatureData, 0);
		temperatureAlarm = temperatureData.value < settings.temperatureMin
				|| temperatureData.value > settings.temperatureMax;

		// Tratamos datos de humedad
		humidityData.value = SHT1X_ReadRelativeHumidity(temperatureData.value);
		LOG("Current relative humidity is %.2f%%", humidityData.value);
		xQueueSend(xQueue, &humidityData, 0);
		humidityAlarm = humidityData.value < settings.humidityMin
				|| humidityData.value > settings.humidityMax;

		stackHWM = uxTaskGetStackHighWaterMark(xTaskGetCurrentTaskHandle());
		LOG("Free stack: %d", stackHWM);

		// Encendemos los leds que indican el estado de las alarmas
		LED_Switch(LEDID_ALARM_TEMPERATURE_MASK, temperatureAlarm);
		LED_Switch(LEDID_ALARM_HUMIDITY_MASK, humidityAlarm);
		//LED_Switch(LEDID_OK_MASK, !temperatureAlarm && !humidityAlarm);

		LED_Switch(LEDID_MEASURING_MASK, false);
		// Esperamos el tiempo necesario
		SLEEP_PasiveMSSleep(settings.interval * 60000);
	}
}
